@@ -16,7 +16,7 @@
 -- and HDBC instances correspond to RDB table schema.
 module Database.HDBC.Query.TH (
   makeRelationalRecord,
-  makeRecordPersistableDefault,
+  makeRelationalRecord',
 
   defineTableDefault',
   defineTableDefault,
@@ -27,10 +27,11 @@ module Database.HDBC.Query.TH (
   inlineVerifiedQuery
   ) where
 
+import Control.Applicative ((<$>), pure, (<*>))
+import Control.Monad (when, void)
 import Data.Maybe (listToMaybe, fromMaybe)
 import qualified Data.Map as Map
-import Control.Applicative ((<$>), (<*>))
-import Control.Monad (when, void)
+import Data.Functor.ProductIsomorphic.TH (reifyRecordType)
 
 import Database.HDBC (IConnection, SqlValue, prepare)
 
@@ -39,17 +40,17 @@ import Language.Haskell.TH.Name.CamelCase (varCamelcaseName)
 import Language.Haskell.TH.Lib.Extra (reportWarning, reportError)
 
 import Database.Record (ToSql, FromSql)
-import Database.Record.TH (recordTemplate, reifyRecordType)
-import Database.Relational.Query
-  (Config, nameConfig, recordConfig, verboseAsCompilerWarning, defaultConfig,
-   Relation, relationalQuerySQL, QuerySuffix)
-import qualified Database.Relational.Query.TH as Relational
+import Database.Record.TH (recordTemplate, defineSqlPersistableInstances)
+import Database.Relational
+  (Config, nameConfig, recordConfig, enableWarning, verboseAsCompilerWarning,
+   defaultConfig, Relation, relationalQuerySQL, QuerySuffix)
+import qualified Database.Relational.TH as Relational
 
 import Database.HDBC.Session (withConnectionIO)
 import Database.HDBC.Record.Persistable ()
 
 import Database.HDBC.Schema.Driver
-  (runLog, newLogChan, takeLogs, Driver, getFields, getPrimaryKey)
+  (foldLog, emptyLogChan, takeLogs, Driver, driverConfig, getFields, getPrimaryKey)
 
 
 defineInstancesForSqlValue :: TypeQ   -- ^ Record type constructor.
@@ -60,19 +61,19 @@ defineInstancesForSqlValue typeCon = do
     |]
 
 -- | Generate all persistable templates against defined record like type constructor.
-makeRelationalRecord :: Name    -- ^ Type constructor name
-                     -> Q [Dec] -- ^ Result declaration
-makeRelationalRecord recTypeName = do
-  rr <- Relational.makeRelationalRecordDefault recTypeName
-  ((typeCon, _), _) <- reifyRecordType recTypeName
-  ps <- defineInstancesForSqlValue typeCon
+makeRelationalRecord' :: Config
+                      -> Name    -- ^ Type constructor name
+                      -> Q [Dec] -- ^ Result declaration
+makeRelationalRecord' config recTypeName = do
+  rr <- Relational.makeRelationalRecordDefault' config recTypeName
+  (((typeCon, avs), _), _) <- reifyRecordType recTypeName
+  ps <- defineSqlPersistableInstances [t| SqlValue |] typeCon avs
   return $ rr ++ ps
 
-{-# DEPRECATED makeRecordPersistableDefault "Use makeRelationalRecord instead of this." #-}
--- | Deprecated. use 'makeRelationalRecord'.
-makeRecordPersistableDefault :: Name    -- ^ Type constructor name
-                             -> Q [Dec] -- ^ Result declaration
-makeRecordPersistableDefault = makeRelationalRecord
+-- | Generate all persistable templates against defined record like type constructor.
+makeRelationalRecord :: Name    -- ^ Type constructor name
+                     -> Q [Dec] -- ^ Result declaration
+makeRelationalRecord = makeRelationalRecord' defaultConfig
 
 -- | Generate all HDBC templates about table except for constraint keys.
 defineTableDefault' :: Config            -- ^ Configuration to generate query with
@@ -83,7 +84,9 @@ defineTableDefault' :: Config            -- ^ Configuration to generate query wi
                     -> Q [Dec]           -- ^ Result declaration
 defineTableDefault' config schema table columns derives = do
   modelD <- Relational.defineTableTypesAndRecord config schema table columns derives
-  sqlvD <- defineInstancesForSqlValue . fst $ recordTemplate (recordConfig $ nameConfig config) schema table
+  sqlvD <- defineSqlPersistableInstances [t| SqlValue |]
+           (fst $ recordTemplate (recordConfig $ nameConfig config) schema table)
+           []
   return $ modelD ++ sqlvD
 
 -- | Generate all HDBC templates about table.
@@ -100,18 +103,17 @@ defineTableDefault config schema table columns derives primary notNull = do
   sqlvD <- defineInstancesForSqlValue . fst $ recordTemplate (recordConfig $ nameConfig config) schema table
   return $ modelD ++ sqlvD
 
--- | Generate all HDBC templates using system catalog informations with specified config.
-defineTableFromDB' :: IConnection conn
-                   => IO conn     -- ^ Connect action to system catalog database
-                   -> Config      -- ^ Configuration to generate query with
-                   -> Driver conn -- ^ Driver definition
-                   -> String      -- ^ Schema name
-                   -> String      -- ^ Table name
-                   -> [Name]      -- ^ Derivings
-                   -> Q [Dec]     -- ^ Result declaration
-defineTableFromDB' connect config drv scm tbl derives = do
+tableAlongWithSchema :: IConnection conn
+                     => IO conn     -- ^ Connect action to system catalog database
+                     -> Config      -- ^ Configuration to generate query with
+                     -> Driver conn -- ^ Driver definition
+                     -> String      -- ^ Schema name
+                     -> String      -- ^ Table name
+                     -> [Name]      -- ^ Derivings
+                     -> Q [Dec]     -- ^ Result declaration
+tableAlongWithSchema connect config drv scm tbl derives = do
   let getDBinfo = do
-        logChan  <-  newLogChan $ verboseAsCompilerWarning config
+        logChan  <-  emptyLogChan
         infoP    <-  withConnectionIO connect
                      (\conn ->
                        (,)
@@ -120,8 +122,14 @@ defineTableFromDB' connect config drv scm tbl derives = do
         (,) infoP <$> takeLogs logChan
 
   (((cols, notNullIdxs), primaryCols), logs) <- runIO getDBinfo
-  mapM_ (runLog reportWarning reportError) logs
-  when (null primaryCols) . reportWarning
+  let reportWarning'
+        | enableWarning config             =  reportWarning
+        | otherwise                        =  const $ pure ()
+      reportVerbose
+        | verboseAsCompilerWarning config  =  reportWarning
+        | otherwise                        =  const $ pure ()
+  mapM_ (foldLog reportVerbose reportWarning' reportError) logs
+  when (null primaryCols) . reportWarning'
     $ "getPrimaryKey: Primary key not found for table: " ++ scm ++ "." ++ tbl
 
   let colIxMap = Map.fromList $ zip [c | (c, _) <- cols] [(0 :: Int) .. ]
@@ -134,6 +142,18 @@ defineTableFromDB' connect config drv scm tbl derives = do
 
   defineTableDefault config scm tbl cols derives primaryIxs (listToMaybe notNullIdxs)
 
+{-# DEPRECATED defineTableFromDB' "Use defineTableFromDB with updated driverConfig field of Driver argument." #-}
+-- | Generate all HDBC templates using system catalog informations with specified config.
+defineTableFromDB' :: IConnection conn
+                   => IO conn     -- ^ Connect action to system catalog database
+                   -> Config      -- ^ Configuration to generate query with
+                   -> Driver conn -- ^ Driver definition
+                   -> String      -- ^ Schema name
+                   -> String      -- ^ Table name
+                   -> [Name]      -- ^ Derivings
+                   -> Q [Dec]     -- ^ Result declaration
+defineTableFromDB' = tableAlongWithSchema
+
 -- | Generate all HDBC templates using system catalog informations.
 defineTableFromDB :: IConnection conn
                   => IO conn     -- ^ Connect action to system catalog database
@@ -142,7 +162,7 @@ defineTableFromDB :: IConnection conn
                   -> String      -- ^ Table name
                   -> [Name]      -- ^ Derivings
                   -> Q [Dec]     -- ^ Result declaration
-defineTableFromDB connect = defineTableFromDB' connect defaultConfig
+defineTableFromDB connect driver = tableAlongWithSchema connect (driverConfig driver) driver
 
 -- | Verify composed 'Query' and inline it in compile type.
 inlineVerifiedQuery :: IConnection conn
