@@ -2,7 +2,7 @@
 
 -- |
 -- Module      : Database.HDBC.Record.Statement
--- Copyright   : 2013-2017 Kei Hibino
+-- Copyright   : 2013-2018 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -16,7 +16,7 @@ module Database.HDBC.Record.Statement (
 
   withUnsafePrepare, withPrepareNoFetch,
 
-  BoundStatement (..), bind', bind, bindTo,
+  BoundStatement (..), bind, bindTo,
 
   ExecutedStatement, executed, result,
 
@@ -28,12 +28,11 @@ module Database.HDBC.Record.Statement (
   ) where
 
 import Control.Exception (bracket)
-import Database.Relational.Query (UntypeableNoFetch (untypeNoFetch))
+import Database.Relational (UntypeableNoFetch (untypeNoFetch))
 import Database.HDBC (IConnection, Statement, SqlValue)
 import qualified Database.HDBC as HDBC
 
-import Database.Record
-  (RecordToSql, ToSql(recordToSql), runFromRecord)
+import Database.Record (ToSql, fromRecord)
 
 -- | Typed prepared statement type.
 newtype PreparedStatement p a =
@@ -47,7 +46,7 @@ data BoundStatement a =
   BoundStatement
   {
     -- | Untyped prepared statement before executed.
-    bound  :: Statement
+    bound  :: !Statement
     -- | Bound parameters.
   , params :: [SqlValue]
   }
@@ -56,9 +55,9 @@ data BoundStatement a =
 data ExecutedStatement a =
   ExecutedStatement
   { -- | Untyped executed statement.
-    executed :: Statement
+    executed :: !Statement
     -- | Result of HDBC execute.
-  , result   :: Integer
+  , result   :: !Integer
   }
 
 -- | Unsafely untype prepared statement.
@@ -103,16 +102,12 @@ withPrepareNoFetch :: (UntypeableNoFetch s, IConnection conn)
 withPrepareNoFetch conn s =
   bracket (prepareNoFetch conn s) finish
 
--- | Typed operation to bind parameters.
-bind' :: RecordToSql SqlValue p -- ^ Proof object to convert from parameter type 'p' into 'SqlValue' list.
-      -> PreparedStatement p a  -- ^ Prepared query to bind to
-      -> p                      -- ^ Parameter to bind
-      -> BoundStatement a       -- ^ Result parameter bound statement
-bind' toSql q p = BoundStatement { bound = prepared q, params = runFromRecord toSql p }
-
--- | Typed operation to bind parameters. Inferred 'RecordToSql' is used.
-bind :: ToSql SqlValue p => PreparedStatement p a -> p -> BoundStatement a
-bind =  bind' recordToSql
+-- | Typed operation to bind parameters. Inferred 'ToSql' is used.
+bind :: ToSql SqlValue p
+     => PreparedStatement p a -- ^ Prepared query to bind to
+     -> p                     -- ^ Parameter to bind
+     -> BoundStatement a      -- ^ Result parameter bound statement
+bind q p = BoundStatement { bound = prepared q, params = fromRecord p }
 
 -- | Same as 'bind' except for argument is flipped.
 bindTo :: ToSql SqlValue p => p -> PreparedStatement p a -> BoundStatement a
@@ -123,7 +118,7 @@ executeBound :: BoundStatement a -> IO (ExecutedStatement a)
 executeBound bs = do
   let stmt = bound bs
   n <- HDBC.execute stmt (params bs)
-  return $ ExecutedStatement stmt n
+  n `seq` return (ExecutedStatement stmt n)
 
 {-# WARNING execute "Use 'executeBound' instead of this. This name will be used for executePrepared function in future release." #-}
 -- | Use 'executeBound' instead of this.
@@ -139,7 +134,7 @@ executePrepared st = executeBound . bind st
 executeBoundNoFetch :: BoundStatement () -> IO Integer
 executeBoundNoFetch = fmap result . executeBound
 
-{- WARNING executeNoFetch "Use 'executeBoundNoFetch' instead of this. This name will be used for runPreparedNoFetch function in future release." -}
+{-# WARNING executeNoFetch "Use 'executeBoundNoFetch' instead of this. This name will be used for runPreparedNoFetch function in future release." #-}
 -- | Use 'executeBoundNoFetch' instead of this.
 --   WARNING! This name will be used for runPreparedNoFetch function in future release.
 executeNoFetch :: BoundStatement () -> IO Integer
